#!/usr/bin/python3
# -*- coding:utf-8 -*-

import os, sys

sys.path.append("@PACKAGE_DIR@")

from module_factory import ModuleFactory
from update_boot import update_boot_params
from modules_desc import security_modules
from utils import print_err

from secure_level import secure_levels

def usage():
	print_err("\n用法: %s LEVEL\n\n"%sys.argv[0])
	print_err("LEVEL 可以是以下选项中的任意一个: \n")
	print_err("  disabled        禁用所有的安全模块\n")
	print_err("  default         仅启用Apparmor\n")
	print_err("  loose           同时启用执行控制和Apparmor\n")
	print_err("  medium          同时启用执行控制和SELinux\n")
	print_err("  strict          同时启用执行控制, SELinux 和三权分立\n")

if len(sys.argv) < 2:
	usage()
	sys.exit(0)

level=sys.argv[1]
if level not in secure_levels.keys():
	print_err("未知的安全级别: %s\n"%sys.argv[1])
	usage()
	sys.exit(1)

modules={"disabled": {"instance": [], "name": []}, "enabled": {"instance": [], "name": []}}

for module in security_modules.keys():
	instance = ModuleFactory.get_instance(module)
	if module not in secure_levels[level]["modules"]:
		mods=modules["disabled"]
	else:
		mods=modules["enabled"]

	mods["instance"].append(instance)
	mods["name"].append(module)

print_err("检查内核配置和软件包安装状态...\n")
for module in modules["enabled"]["instance"]:
	if not module.check_kernel():
		print_err("%s: 当前内核不支持\n"%module.name)
		sys.exit(1)

	ret, pkg_list=module.check_packages(modules["enabled"]["name"])
	if not ret:
		print_err("%s: 以下软件包未安装或版本不匹配:\n  %s\n"%(module.name, "\n  ".join(pkg_list)))
		sys.exit(1)

	ret, file_list=module.check_files(modules["enabled"]["name"])
	if not ret:
		print_err("%s: 以下文件未安装或版本不匹配:\n  %s\n"%(module.name, "\n  ".join(file_list)))
		sys.exit(1)

print_err("为禁用的安全模块执行安全状态切换前操作...\n")
for module in modules["disabled"]["instance"]:
	result, error = module.pre_disable()
	if not result:
		print_err("%s: %s\n"%(module.name, error))
		sys.exit(1)

print_err("为启用的安全模块执行安全状态切换前操作...\n")
for module in modules["enabled"]["instance"]:
	result, error = module.pre_enable()
	if not result:
		print_err("%s: %s\n"%(module.name, error))
		sys.exit(1)

print_err("更新系统启动配置选项...\n")
params = secure_levels[level]["args"]
result, error = update_boot_params(params)
if not result:
	print_err("更新系统启动配置选项时出错: %s\n"%error)
	sys.exit(1)

print_err("为禁用的安全模块执行安全状态切换后操作...\n")
for module in modules["disabled"]["instance"]:
	result, error = module.post_disable()
	if not result:
		print_err("%s: %s\n"%(module.name, error))
		sys.exit(1)

print_err("为启用的安全模块执行安全状态切换后操作...\n")
for module in modules["enabled"]["instance"]:
	result, error = module.post_enable()
	if not result:
		print_err("%s: %s\n"%(module.name, error))
		sys.exit(1)
