#!/usr/bin/python2
# -*- coding:utf-8 -*-

import os

from module_factory import ModuleFactory
from update_boot import update_boot_params
from modules_desc import security_modules
from exectl_module import ExectlModule
from selinux_module import SELinuxModule
from apparmor_module import ApparmorModule
from admin3_module import Admin3Module

from utils import *

MODE_DISABLED=0
MODE_ENABLED=1
MODE_ENFORCE=2
MODE_SOFTMODE=4

secure_levels={
"default":  {  "modules": {"exectl": MODE_ENFORCE},
		"args": "security=kysec kysec_status=2 kysec_3adm=0"
	    },
"strict":   {
		"modules": {"selinux": MODE_ENFORCE, "exectl": MODE_ENFORCE, "admin3": MODE_ENABLED},
		"args": "security=selinux,kysec kysec_status=2 kysec_3adm=1 selinux=1 enforcing=1"
	    },
"custom":   {"modules": {}, "args": "security="}
}

SECURITY_CONFIG="/etc/security.conf"
CUSTOM_CONFIG="/etc/security_custom.conf"
CUSTOM_MODE="custom"

def get_valid_security_module_name():
	module_name = ""
	modules = security_modules.keys()
	for module in modules:
		module_name = module + " " + module_name
	return module_name

def custom_modules_add(modules):
	c_args = "security="
	c_modules = {}
	m_selinux = False
	m_kysec = False
	m_3adm = False
	module_check = 0

	if not len(modules):
		print_err("错误：请求设置的安全模块列表为空！\n\n");
		return False

	for module_name in modules:
		if module_name == "apparmor":
			c_modules["apparmor"] = MODE_ENABLED
			c_args = "%sapparmor,"%c_args
		elif module_name == "selinux":
			m_selinux = True
			c_modules["selinux"] = MODE_ENFORCE
			c_args = "%sselinux,"%c_args
		elif module_name == "exectl":
			m_kysec = True
			c_modules["exectl"] = MODE_ENFORCE
			c_args = "%skysec,"%c_args
		elif module_name == "admin3":
			m_3adm = True
			c_modules["admin3"] = MODE_ENABLED
		else:
			print_err("错误：%s模块不是一个合法的安全模块！\n"%module_name)
			return False

	if "apparmor" in modules and "selinux" in modules:
		print_err("系统不支持同时开启selinux和apparmor.")
		return False

	c_args = c_args[:-1]

	if m_selinux:
		c_args = c_args + " selinux=1 enforcing=1"
	if m_kysec:
		c_args = c_args + " kysec_status=2"
	if m_3adm:
		if not m_kysec:
			print_err("错误：在选择三权分立的情况下必须同时选择执行控制模块！\n")
			return False
		c_args = c_args + " kysec_3adm=1"
	else:
		if m_kysec:
		    c_args = c_args + " kysec_3adm=0"
	secure_levels[CUSTOM_MODE]["modules"] = c_modules
	secure_levels[CUSTOM_MODE]["args"] = c_args
	return True

def security_level_isvalid(level):
	if level not in secure_levels.keys():
		return False
	return True

def security_level_set(level, auto_resetpw=False, rollback=False, module_list=[]):
	modules={"disabled": {"instance": [], "name": []}, "enabled": {"instance": [], "name": []}}

	if level == CUSTOM_MODE:
		if rollback:
			print_err("回退到上次的自定义模式\n")
			result, c_modules = read_custom_mode(CUSTOM_CONFIG)
			if not result:
				print_err("Failed\n")
				print_err("读取用户自定义配置时出错: %s\n"%c_modules[0])
				return False
		else:
			c_modules = module_list

		result = custom_modules_add(c_modules)
		if not result:
			return False

	for module in security_modules.keys():
		instance = ModuleFactory.get_instance(module, auto_resetpw)
		if module not in secure_levels[level]["modules"].keys():
			mods=modules["disabled"]
		else:
			mods=modules["enabled"]
	
		mods["instance"].append(instance)
		mods["name"].append(module)

	for module in modules["enabled"]["instance"]:
		print_err("正在为 '%s' 检查内核配置... "%module.desc)
		if not module.check_kernel():
			print_err("Failed\n")
			print_err("%s: 当前内核不支持\n"%module.desc)
			return False
		print_err("OK\n")
	
		print_err("正在为 '%s' 检查软件包安装状态... "%module.desc)
		ret, pkg_list=module.check_packages(modules["enabled"]["name"])
		if not ret:
			print_err("Failed\n")
			print_err("%s: 以下软件包未安装或版本不匹配:\n  %s\n"%(module.desc, "\n  ".join(pkg_list)))
			return False
		print_err("OK\n")

		print_err("正在为 '%s' 检查文件安装状态... "%module.desc)
		ret, file_list=module.check_files(modules["enabled"]["name"])
		if not ret:
			print_err("Failed\n")
			print_err("%s: 以下文件未安装或版本不匹配:\n  %s\n"%(module.desc, "\n  ".join(file_list)))
			return False
		print_err("OK\n")

		print_err("正在为开启 '%s' 检查系统配置状态... "%module.desc)
		ret, error=module.check_system(True)
		if not ret:
			print_err("Failed\n")
			print_err("%s: 系统配置不满足:\n  %s\n"%(module.desc, error))
			return False
		print_err("OK\n")

	for module in modules["disabled"]["instance"]:
		print_err("正在为关闭 '%s' 检查系统配置状态... "%module.desc)
		ret, error=module.check_system(False)
		if not ret:
			print_err("Failed\n")
			print_err("%s: 系统配置不满足\n%s\n"%(module.desc, error))
			return False
		print_err("OK\n")

	for module in modules["enabled"]["instance"]:
		print_err("正在启用 '%s' ... "%module.desc)
		result, error = module.do_enable()
		if not result:
			print_err("Failed\n")
			print_err("%s: %s\n"%(module.desc, error))
			return False
		if module.desc == "三权分立":
			os.system("sed -i 's/\/home\/secadm:\/usr\/sbin\/nologin/\/home\/secadm:\/bin\/bash/g' /etc/passwd 2> /dev/null")
			os.system("sed -i 's/\/home\/auditadm:\/usr\/sbin\/nologin/\/home\/auditadm:\/bin\/bash/g' /etc/passwd 2> /dev/null")
		print_err("OK\n")

	for module in modules["disabled"]["instance"]:
		print_err("正在禁用 '%s' ... "%module.desc)
		result, error = module.do_disable()
		if not result:
			print_err("Failed\n")
			print_err("%s: %s\n"%(module.desc, error))
			return False
		if module.desc == "三权分立":
			os.system("sed -i 's/\/home\/secadm:\/bin\/bash/\/home\/secadm:\/usr\/sbin\/nologin/g' /etc/passwd 2> /dev/null")
			os.system("sed -i 's/\/home\/auditadm:\/bin\/bash/\/home\/auditadm:\/usr\/sbin\/nologin/g' /etc/passwd 2> /dev/null")
		print_err("OK\n")

	print_err("正在更新系统启动配置... ")
	params = secure_levels[level]["args"]
	result, error = update_boot_params(params)
	if not result:
		print_err("Failed\n")
		print_err("更新系统启动配置时出错: %s\n"%error)
		return False
	print_err("OK\n")

	print_err("正在写入系统安全配置... ")
	result, error = write_config(SECURITY_CONFIG, level)
	if not result:
		print_err("Failed\n")
		print_err("写入系统安全配置时出错: %s\n"%error)
		return False
	print_err("OK\n")

	return True

def security_level_get():
	modules=[]
	mode = None
	global last_custom

	for module_name in security_modules.keys():
		module = ModuleFactory.get_instance(module_name, False)
		if module.getstatus():
			modules.append(module_name)

	for level in secure_levels.keys():
		level_modules = list(secure_levels[level].get("modules").keys())
		if sorted(modules) == sorted(level_modules):
			mode = level
			break
		else:
			mode = CUSTOM_MODE

	if mode == CUSTOM_MODE:
		print("\n当前安全级别 : 用户自定义(custom)\n")
		result, error = write_custom_mode(CUSTOM_CONFIG, modules)
		if not result:
			print_err("Failed\n")
			print_err("写入用户自定义配置时出错: %s\n"%error)
			return False
	else:
		print("\n当前安全级别 : %s"%mode)
	print("-------------------------------------------------")
	print("  安全模块     |     当前状态     |  默认状态")
	print("-------------------------------------------------")

	result = custom_modules_add(modules)
	if not result:
		return False

	for module in security_modules.keys():
		instance = ModuleFactory.get_instance(module, False)
		class_val = instance.__class__
		status   = class_val.getstatus_desc(class_val.status_desc, class_val.getstatus())
		if mode:
			if module in secure_levels[mode]["modules"].keys():
				default_status = secure_levels[mode]["modules"].get(module)
			else:
				default_status = MODE_DISABLED

			if default_status == MODE_DISABLED:
				continue
			default_status = class_val.getstatus_desc(class_val.status_desc, default_status)
		else:
			default_status = "未知"

		print("%s        %-20s%-18s"%(instance.desc, status, default_status))
